/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Alfresco Plugin
 *
 * Author: Sebastian Kummer <s.kummer@zarafa.com>
 *
 */

Pluginalfresco.prototype = new Plugin;
Pluginalfresco.prototype.constructor = Pluginalfresco;
Pluginalfresco.superclass = Plugin.prototype;

function Pluginalfresco(){}


/**
 * init
 *
 * registers for events 
 */
Pluginalfresco.prototype.init = function(){
    this.registerHook("client.module.maillistmodule.contextmenu.buildup");
    this.registerHook("client.module.maillistmodule.execute.before");
    // use the sharedFoldersPane to start Alfresco preLoad stuff
    this.registerHook("client.module.hierarchymodule.sharedFoldersPane.buildup");
}


/**
 * PLUGIN execute
 *
 * is called when registered events are triggered 
 */
Pluginalfresco.prototype.execute = function(eventID, data){
    switch(eventID){
        case "client.module.maillistmodule.contextmenu.buildup":
            this.addAlfrescoArchiveContextMenu(data);
            break;
        case "client.module.maillistmodule.execute.before":
            this.mailListModuleExecute(data);
            break;
        case "client.module.hierarchymodule.sharedFoldersPane.buildup":
            this.preloadAlfrescoData(data);
            break;
    }
}


/**
 * Event: preloadAlfrescoData
 *
 * invokes the "alfrescospacemodule" and calls preloadAlfrescoDate() which 
 * gets all nodes from alfresco and caches them on the server
 */
Pluginalfresco.prototype.preloadAlfrescoData = function(data){
    setTimeout(function(){
        eval("asm = new alfrescospacemodule();");
        if (asm) {
            var asmID = webclient.addModule(asm);
            asm.init(asmID);
            asm.preloadAlfrescoData();
        }
    },10000);
}


/**
 * Event: addArchiveContextMenu
 *
 * adds new entries to the contextmenu
 */
Pluginalfresco.prototype.addAlfrescoArchiveContextMenu = function(data){
    var items = data["contextmenu"];
    items.push(webclient.menu.createMenuItem("seperator", ""));        
    items.push(webclient.menu.createMenuItem("alfresco", dgettext('plugin_alfresco', "Archive to Alfresco")+"...", dgettext('plugin_alfresco', "Archive to Alfresco"), Pluginalfresco.prototype.eventListContextMenuArchiveFiles));
}


/**
 * Event: eventListContextMenuAlfrescoSpaceDialog
 *
 * action for archive to alfresco (open Space selector)
 */
Pluginalfresco.prototype.eventListContextMenuAlfrescoSpaceDialog = function(moduleObject, element, event) {
    element.parentNode.style.display = "none";
    // open dialog to select space
    webclient.openModalDialog(moduleObject, 'alfrescospace', DIALOG_URL+"task=alfrescospace&plugin=alfresco", 600,380, alfrescoSpaceDialog_archiveToCallBack, {moduleObject:moduleObject, element:element, elementid:element.parentNode.elementid});
}


/**
 * Callback: alfrescoSpaceDialog_archiveToCallBack
 *
 * Callback functions when selecting a space to archive to
 * contacts server to start archivation
 */
function alfrescoSpaceDialog_archiveToCallBack(result, callbackData) {

    var data = new Array();
    data["store"] = callbackData.moduleObject.storeid;
    data["parententryid"] = callbackData.moduleObject.entryid;
    // add one entryid to satisfy the API when passing through the maillistmodules execute
    data["entryid"] = callbackData.moduleObject.entryids[callbackData.elementid];    
    data["destpath"] = result.nodelink;

    var entryids = Array();

    var items = callbackData.moduleObject.getSelectedMessages(callbackData.elementid);
    for(i in items){
        entryids.push(callbackData.moduleObject.entryids[items[i]]);
    }
    data["entryids"] = entryids;

    webclient.xmlrequest.addData(callbackData.moduleObject, "AlfrescoArchive", data);

    // deselect selected message
    eventListCheckSelectedContextMessage(callbackData.moduleObject);
}


/**
 * Action: mailListModuleExecute
 *
 * process incoming execute events for maillistmodule
 */
Pluginalfresco.prototype.mailListModuleExecute = function(data) {
    var type = data["type"];
    var action = data["action"];

    switch(type){
        case "AlfrescoArchiveResponse":
            this.archiveResponse(action);
            break;
    }
}


/**
 * Action: archiveResponse
 *
 * process responses from the Alfresco server
 */
Pluginalfresco.prototype.archiveResponse = function(action) {
    var archivedAttachments = dhtml.getXMLValue(action, "archivedAttachments", "none");
    var stored = dhtml.getXMLValue(action, "stored", "none");
    alert(dgettext('plugin_alfresco', "Archived files to Alfresco")+': '+archivedAttachments+"\n"+stored);
}


/**
 * Event: eventListContextMenuArchiveFiles
 *
 * action for archive to alfresco (open Space selector)
 */
Pluginalfresco.prototype.eventListContextMenuArchiveFiles = function(moduleObject, element, event) {
    element.parentNode.style.display = "none";
    // open dialog to select the files
    webclient.openModalDialog(moduleObject, 'filestoarchive', DIALOG_URL+"task=filestoarchive&plugin=alfresco&storeid=" + moduleObject.storeid + "&parententryid=" + moduleObject.entryid + "&entryid=" + moduleObject.entryids[element.parentNode.elementid], 600, 700, alfrescoFilesToArchive_filesToCallBack, {moduleObject:moduleObject, element:element, elementid:element.parentNode.elementid});
}



/**
 * Callback: alfrescoSpaceDialog_archiveToCallBack
 *
 * Callback functions when selecting a space to archive to
 * contacts server to start archivation
 */
function alfrescoFilesToArchive_filesToCallBack(result, callbackData) {
    var data = new Array();
    var ids = result.pop();
    data["node"] = ids["node"];
    data["entryid"] = ids["entryid"];
    data["parententryid"] = ids["parententryid"];
    data["store"] = ids["storeid"];
    data["fullpath"] = ids["fullpath"];
    data["attachments"] = result; 
    webclient.xmlrequest.addData(callbackData.moduleObject, "AlfrescoArchive", data);

}


// ----------------------------- FILES FROM ALFRESCO --------------------------------


/**
 * Callback: alfrescoLinkDialog_selectCallBack
 *
 * insert an Alfresco file link to the email body
 */
function alfrescoLinkDialog_selectCallBack(result, callbackData) {
    // the message was changed
    setMessageChanged();
    
    var htmlFormat = dhtml.getElementById("use_html").value == "true" ? true : false;
    var htmlBody = dhtml.getElementById("html_body");
    var cursorPos = webclient.settings.get("createmail/cursor_position", "start");
        
    if(htmlFormat == true) {
        // FCKEditor is installed
        if(typeof(FCKeditorAPI) != "undefined" && (fckEditor = FCKeditorAPI.GetInstance("html_body"))) {
            fckEditor.InsertHtml(result.htmllink);
        }
    } else {
        // FCKEditor is not installed
        if(cursorPos == "start") {
            htmlBody.value += result.link;
        } else {
            if(htmlBody && typeof htmlBody == "object") {
                // TODO add link the correct way (!!!???)
                htmlBody.value += result.link;
//                currentPos = htmlBody.value.length;
//                dhtml.setSelectionRange(htmlBody, currentPos, currentPos);
            }
        }
    }
}


/**
 * Callback: alfrescoAttachmentDialog_selectCallBack
 *
 * downloads and inserts an Alfresco file to the attachments of an e-mail
 */
function alfrescoAttachmentDialog_selectCallBack (result, callbackData) {
    dhtml.getElementById('alfrescofullpath').value = result.nodelink;
    dhtml.getElementById('alfrescolabel').value = result.nodelabel;

    // submits the upload form -> alfresco server side plugin will be triggered 
    dhtml.getElementById('upload').submit();
}
